%
% This is the second encoder script for
%
%   c64mp3 by Pex 'Mahoney' Tufvesson, February 2010
%
% Run it using Matlab, you'll probably need a couple of toolboxes
% and a lot of patience.
% After running all matlab scripts, you'll find a number of
% files named "SID*.s".
% Copy these into the decoder folder.
%
% Want to know more?  http://mahoney.c64.org
% Want to hear C64 a cappella?  Visa Röster at http://www.livet.se/visa
% Want a happy life? Do something _not_ requiring electricity! :-D
%

format long;
format compact;
clear;

loadScoreMatrix = 1;

% With all these waveforms lying around, we need to find those that
% actually matters.
% Let's score every one of them, by comparing it to all the others.
% First, delete the "silent" ones. These have SIDVoiceVolume=0.
% Then, put them all in a big high-score table.
% Choose the one with the highest points.
% And, "delete" all waveforms that are "almost" identical by some
% magical threshold that I'll need to make a few iterations before I can
% determine what it is.
% Repeat, until all waveform memory is taken.
% All waveforms that didn't make it... too bad
% I need a cross-coherence matrix with all waveforms compared to all other.
% From this, make a cross correlation vector which is the score for every
% waveform "how equal it is to all the others".
% When selecting a waveform, delete all the ones that are "equal" to it -
% by removing them from the score matrix.
% And then select the next waveform.

% Correlation will be better if we normalize all waveforms to +1.0 at the
% peak.
% We could choose to do this on 7812.5Hz-waveforms, 51 bytes long.
% Or we could do it with 300-word 44100Hz floating point waveforms.
% Ah, the computer feels fast today, let's use the 44100Hz waveforms!

% ToDo: cross correlate waveforms. The score matrix is too expensive.
% Just calculate the score vector directly. Or, we need to "clean out"
% the uninteresting waveforms before we come here.

%mean(SIDVoiceVolume) = 0.618678032953750
%mean(SIDVoiceVolume) * nofSlices = 9725
% We would only need a scoreMatrix = zeros(9725,9725) = 94Mcorrelations 
% instead of (15719,15719) = 250Mcorrelations
%zeros(9800,9800,'int8'); is ok
%zeros(9800,9800,'int16'); is ok
%zeros(9800,9800,'single'); is not ok - out of memory
% We could "clear out" a couple of non-used variables as well.
% We don't need the "waveforms" table until later.
% Maybe even "dump" used variables to disk, do a "clear", and
% then re-read those we want to use - to defragment virtual memory space.
% I'm running win32 here, so there's only 4GB contiguous virtual memory to use.
% After a "clear", zeros(9800,9800,'single'); is ok

% SIDVoiceVolume cannot be used here, since the waveformsNormalized only
% contains waveforms where volume > 0.


load 'waveformsNormalized.mat' waveformsNormalized
nofWaveforms = size(waveformsNormalized);
nofWaveforms = nofWaveforms(1);
scoreMatrix = zeros(nofWaveforms,nofWaveforms,'single');

waveformStatus = zeros(1,nofWaveforms); 
                                      % 0: still left
                                      %-1: silent (not used anymore)
                                      % X: chosen as/similar to waveformX

if (loadScoreMatrix == 1)
    load 'scoreMatrix.mat'
else
    for waveformNo=1:nofWaveforms-1,
    %for waveformNo=1067:1068,
        waveformNo
        if (waveformStatus(waveformNo) == 0)
            for compareNo=waveformNo:nofWaveforms-1,
                if (waveformStatus(compareNo) == 0)
                    % What is a score?   sum(wave1 .* wave2) seems like a bad
                    % idea...
                    % Correlating frequency spectra (ignoring phase) would
                    % be a good idea. But, since the "source" of the sound
                    % comes from impulses of the vocal cords, the material
                    % is pretty well aligned when it comes to the phase of
                    % the higher overtones.
                    % Maybe   sum((wave1 - wave2)^2)
    %                score=sum(waveformsNormalized(waveformNo,:) .* waveformsNormalized(compareNo,:));
    % worst case sum = 300 * (1 -- 1)^2 = 300 * 2^2 = 1200
                    score = 2000 - sum((waveformsNormalized(waveformNo,:) - waveformsNormalized(compareNo,:)).^2);
                    scoreMatrix(waveformNo,compareNo) = score;
                    scoreMatrix(compareNo,waveformNo) = score; % Make it symmetric
                end
            end
        end
    end
    save 'scoreMatrix.mat' scoreMatrix
end

figure(1)
clf
plot(scoreMatrix(1067,:));
hold on;



% Remove silly values from scoreMatrix.
% We need to steer away from BadThings(tm) +Infs -Infs in the scoreMatrix
scoreMatrix = max(scoreMatrix, -1); % Let all negative values become -1.0
scoreMatrix = min(scoreMatrix, 2001); % Let all (probably none) values clamp at 2001

% If we wanted to do the scoreMatrix a triangular matrix, then do this:
%scoreMatrix = tril(scoreMatrix); %Erase upper right values above the diagonal
% ...but we don't want to do that. ;)

chosenWaveforms = 0;
whichWaveformsAreChosen = zeros(1,nofWaveforms);
howManyWaveformsAreChosen = zeros(1,nofWaveforms);
residualScore = zeros(1,nofWaveforms);
waveformsLeft = nofWaveforms;
threshold = 1999.5;
thresholdHistory = zeros(1,nofWaveforms);
waveformsLeftHistory = zeros(1,nofWaveforms);

% Could probably choose more waveforms for the first ones, and then less
% and less for the coming ones...

% Make a function that takes waveformNo as input (0..254),
% and returns "howMany" to choose.
% The sum of the function should be nofWaveforms = ~10000

while ((chosenWaveforms < 254) && (waveformsLeft >= 1) && (threshold > 1500) && (waveformsLeft > (255-chosenWaveforms)))
%This is one way of choosing how many waveforms to choose:
%    howManyToChoose = (1.0 - (chosenWaveforms) / 260) * 72

% But, actually this one sounds better:    
    howManyToChoose = 37;


% One version of the selection algorithm. Not the best one:    
%     % Sum the score matrix to form a score vector.
%     scoreVector = sum(scoreMatrix,1);
%     % Plot "residual score" (the sum of the score vector) after every
%     % iteration. This will give us a clue of "when to stop", depending on what
%     % our "delete-this-because-it-is-too-similar-to-one-we've-chosen"-threshold
%     % is.
%     % We need to steer away from BadThings(tm) +Infs -Infs in the scoreMatrix
%     residualScore(chosenWaveforms+1) = sum(max(min(scoreVector,2001),0));
%     
%     % scoreVector now contains "how well do I fit with the other waveforms"
%     % so, select the best-fitting one:
%     % Select the waveform with the highest score. Keep track of where
%     % it came from.
%     [maxScore, chosenWaveform] = max(scoreVector);
%     whichWaveformsAreChosen(chosenWaveforms+1) = chosenWaveform;
%     chosenWaveforms = chosenWaveforms + 1
%     
%     % Remove this, and waveforms "similar" to this.
%     % Threshold 1850 -> Finds only  30 waveforms
%     % Threshold 1875 -> Finds only  60 waveforms
%     % Threshold 1900 -> Finds only 110 waveforms
%     % Threshold 1920 -> Finds only 220 waveforms
%     % Threshold 1925 -> Finds only 260 waveforms
%     % Threshold 1940 -> Finds only 450 waveforms
%     % Threshold 1943 -> Finds only 500 waveforms
%     % Threshold 1950 -> End with a residualScore at 500 waveforms
%     threshold = 1940; % ToDo: choose a better one
%     howMany = 0;
%     
%     for waveformNo = 1:nofWaveforms,
%         if (scoreMatrix(chosenWaveform,waveformNo) >= threshold) % This hopefully includes the waveform itself ;)
%            scoreMatrix(waveformNo,:) = 0;
%            scoreMatrix(:,waveformNo) = 0;
%            waveformStatus(waveformNo) = chosenWaveform;
%            howMany = howMany + 1;
%         end
%     end
%    
%     howManyWaveformsAreChosen(chosenWaveforms) = howMany;
%     
%     % ToDo: Choose a better selection strategy. The above will select a
%     % number of waveforms that are "unique". This is not good, we should
%     % ideally choose waveforms that are used a multiple of times.
    

    
    
    
    % Start with zeroing out the diagonal in the scoreMatrix. It's either 0
    % for ???
    
    % Problem is that the scoring algorithm can see that "this is a good
    % one" - but the removing algorithm will only erase a few ones.
    
    % Set a threshold. And count the number of matches in the scoreMatrix
    % that is above this threshold. Choose the waveform with the most
    % matches. And erase those matches.
    % Then repeat, maybe changing the threshold underways. This will give
    % more "hits" per chosen waveform, I think.

    
     % Count the number of scores above threshold for a waveform:
%     scoreVector = zeros(1,nofWaveforms);
%     for waveformNo = 1:nofWaveforms,
%         scoreVector(waveformNo) = sum((scoreMatrix(waveformNo) > threshold),1);
%     end
% Easier:
    howMany = 0;
    while (howMany < howManyToChoose)
        scoreVector = sum((scoreMatrix >= threshold),1);
        [howMany, chosenWaveform] = max(scoreVector);
        if (howMany < howManyToChoose)
            threshold = threshold - 0.4;
        end
    end
    whichWaveformsAreChosen(chosenWaveforms+1) = chosenWaveform;
    chosenWaveforms = chosenWaveforms + 1
    threshold
    waveformsLeft
    howMany

    % Mark all waveforms similar to this for deletion. But don't delete
    % them yet, since this will destroy scores that we were about to find
    % later in the for-loop.
    for waveformNo = 1:nofWaveforms,
        if (scoreMatrix(chosenWaveform,waveformNo) >= threshold) % This hopefully includes the waveform itself ;)
           if (waveformStatus(waveformNo) == 0)
               waveformStatus(waveformNo) = -chosenWaveform;
               waveformsLeft = waveformsLeft - 1;
           end
        end
    end
    
    % Remove all waveforms that we've chosen above.
    for waveformNo = 1:nofWaveforms,
        if (waveformStatus(waveformNo) < 0)
           waveformStatus(waveformNo) = abs(waveformStatus(waveformNo));
           scoreMatrix(waveformNo,:) = -1;
           scoreMatrix(:,waveformNo) = -1;
        end
    end
    
    
%     
%     % ToDo: Choose a better selection strategy. The above will select a
%     % number of waveforms that are "unique". This is not good, we should
%     % ideally choose waveforms that are used a multiple of times.
    howManyWaveformsAreChosen(chosenWaveforms) = howMany;
    thresholdHistory(chosenWaveforms) = threshold;
    waveformsLeftHistory(chosenWaveforms) = waveformsLeft;
end

% Now, choose the ones that are left:
while ((chosenWaveforms < 254) && (waveformsLeft >= 1))
    scoreVector = sum((scoreMatrix >= threshold),1);
    [howMany, chosenWaveform] = max(scoreVector);
    whichWaveformsAreChosen(chosenWaveforms+1) = chosenWaveform;
    chosenWaveforms = chosenWaveforms + 1
    threshold
    waveformsLeft
    howMany

    % Mark all waveforms similar to this for deletion. But don't delete
    % them yet, since this will destroy scores that we were about to find
    % later in the for-loop.
    waveformNo = chosenWaveform;
    if (waveformStatus(waveformNo) == 0)
        waveformStatus(waveformNo) = -chosenWaveform;
        waveformsLeft = waveformsLeft - 1;
    end
    
    % Remove all waveforms that we've chosen above.
    for waveformNo = 1:nofWaveforms,
        if (waveformStatus(waveformNo) < 0)
           waveformStatus(waveformNo) = abs(waveformStatus(waveformNo));
           scoreMatrix(waveformNo,:) = -1;
           scoreMatrix(:,waveformNo) = -1;
        end
    end
    howManyWaveformsAreChosen(chosenWaveforms) = howMany;
    thresholdHistory(chosenWaveforms) = threshold;
    waveformsLeftHistory(chosenWaveforms) = waveformsLeft;
end



figure(2);
clf;
subplot(3,1,1);
plot(howManyWaveformsAreChosen(1:chosenWaveforms));
title('howManyWaveformsAreChosen');
subplot(3,1,2);
plot(thresholdHistory(1:chosenWaveforms));
title('thresholdHistory');
subplot(3,1,3);
plot(waveformsLeftHistory(1:chosenWaveforms));
title('waveformsLeftHistory');

figure(3);
plot(waveformStatus);


% When we have reached ~256 waveforms, stop.

save 'whichWaveformsAreChosen.mat' whichWaveformsAreChosen
save 'waveformStatus.mat' waveformStatus




% Now, we have everything to "simulate" playback.
% SIDVoiceVolume tells us when to sound, and when to be quiet.
% Every time SIDVoiceVolume == 1, increase waveformNo
% Use waveformStatus to check which waveform we shall play.
% Increase pointer into SIDVoiceVolume every 8.192ms.
% This will be a bad "sample&hold" of waveforms, no interpolation.
% But I will make a Matlab model of "how bad" this will sound,
% just to get an idea.





% When all waveforms are chosen, it's time to try to map _every_ single
% waveform that we have onto a linear combination of two waveforms.
% Maybe this will be with some heavy limitations due to memory
% requirements.
% And also, the number of volume levels will be only a few, probably
% only 25%, 33%, 50%, 66%, 75% and 100%.



% Let's try to create one waveform and modify this.

% The replay routine will have to take care of the pitching.
% And no pitch tables, we need to have "human" qualities of the
% pitch curve, so it needs to be imperfect.

% The mixing routine will blend "the old waveform" with a new one.
% The mixing routine will use pha to store values.
% The mixing routine cannot use pla to get values, since any
% interrupt that occurs will destroy the waveform.

% Or, the waveform could be stored, double buffered, in ZeroPage.


% Replay with no mixing routine:

% IRQ handling                    7
%         STA     SaveAnmi+1      3
%         ...a few clocks sync 0 - 4 of them
%         JMP     ZeroCode        3
%ZeroCode:
%        LDA     #$11             2
%        STA     $D404            4
%        LDA     #9               2
%        STA     $D404            4
%Poi:
%        LDA     $3000            4
%        STA     $D401            4
%        LDA     #1               2
%        STA     $D404            4
%        INC     Poi+1            5
%SaveAnmi:
%        LDA     #$A              2
%        JMP     $DD0C            3
%DD0C:   RTI                      6
%                               =55-59 clocks in all
% We have 126-55 = 67-71 clocks left per sample for the mixing routine
% A badline will take 40 of these. = 27-31 clocks left on a badline

